package com.gic.haoban.manage.web.exception;

import com.gic.haoban.base.api.common.ShowFrontMessageException;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.web.controller.WebBaseController;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created 2018/7/24.
 *
 * @author hua
 */
//@ControllerAdvice
public class GlobalExceptionHandler extends WebBaseController {
    private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    private static final Pattern pattern = Pattern.compile("(.*: )(.*)(\\n.*)");


    @ResponseBody
    @ExceptionHandler(ShowFrontMessageException.class)
    public HaobanResponse ControllerException(HttpServletResponse response, ShowFrontMessageException ex) {
        HaobanResponse res = new HaobanResponse();
        res.setErrorCode(HaoBanErrCode.ERR_0.getCode());
        res.setMessage(ex.getMessage());
        return res;
    }


    @ResponseBody
    @ExceptionHandler(RuntimeException.class)
    public HaobanResponse ControllerException(HttpServletResponse response, RuntimeException ex) {
        HaobanResponse res = new HaobanResponse();
        res.setErrorCode(HaoBanErrCode.ERR_3.getCode());
        String message = ex.getMessage();
        if (message != null && message.indexOf(ShowFrontMessageException.class.getName()) > -1) {
            Matcher matcher = pattern.matcher(message);
            while (matcher.find()) {
                message = (matcher.group(2));
            }
        } else {
            message = HaoBanErrCode.ERR_3.getMsg();
        }
        res.setMessage(message);
        return res;
    }

    @ResponseBody
    @ExceptionHandler(Exception.class)
    public HaobanResponse ControllerException(HttpServletResponse response, Exception ex) {
        logger.error("err");
        ex.printStackTrace();
        HaoBanErrCode errCode = HaoBanErrCode.ERR_3;
        StringBuilder sb = new StringBuilder();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (PrintWriter printWriter = new PrintWriter(baos)) {
            ex.printStackTrace(printWriter);
        }
        try {
            sb.append(baos.toString());
        } catch (Exception ignored) {
        }
        if (sb.length() == 0) {
            sb.append(ex.getMessage());
        }

        // 输出详细错误信息，便于调试
        return resultResponse(errCode, null, sb.toString());
    }


    @ResponseBody
    @ExceptionHandler(NoLoginException.class)
    public HaobanResponse ControllerException(HttpServletResponse response, NoLoginException ex) {
        return resultResponse(HaoBanErrCode.ERR_4);
    }

}
